SystemOrganization addCategory: #'Pier-Counter'!

WAFileLibrary subclass: #PRCounterScripts
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Counter'!

!PRCounterScripts methodsFor: 'files' stamp: 'lr 10/1/2008 11:44'!
counterJs
	^ 'var Counter = Class.create({
	initialize: function(element) {
		this.element = $(element);
		this.options = Object.extend({
			default: "#{days} days #{hours} hours #{minutes} minutes #{seconds} secons",
			expired: "Done",
			duration: 3600,
			interval: 1,
			sync: 60,
			url: null
		}, arguments[1] || {});
		this.default = new Template(this.options.default);
		this.expired = new Template(this.options.expired);
		this.start(this.options.duration);
	},
	start: function(duration) {
		if (!!this.executor) {
			this.duration = duration;
			this.sync = this.options.sync;
			this.executor = new PeriodicalExecuter(this.update.bind(this), this.options.interval);
			this.update();
		}
		return this;
	},
	stop: function() {
		if (this.executor) {
			this.executor.stop();
			this.executor = null;
		}
		return this;
	},	
	
	// private
	age: function(num1, num2) {
		var value = Math.floor(Math.abs(this.duration) / num1);
		return num2 ? value % num2 : value;
	},
	update: function() {
		this.duration -= this.options.interval;
		if (this.options.url) {
			this.sync -= this.options.interval;
			if (this.sync < 0) {
				this.stop();
				new Ajax.Request(this.options.url, {
					method: "get",
					onSuccess: function(transport) {
						this.start(transport.responseText);
					}.bind(this)
				});
				return this;
			}
		}
		this.element.update((this.duration < 0 ? this.expired : this.default).evaluate({
			days: this.age(86400, 7), totalDays: this.age(86400),
			hours: this.age(3600, 24), totalHours: this.age(3600),
			minutes: this.age(60, 60), totalMinutes: this.age(60),
			seconds: this.age(1, 60), totalSeconds: this.age(1)
		}));
		return this;
	}
});'! !

!PRCounterScripts methodsFor: 'accessing' stamp: 'lr 9/30/2008 13:58'!
selectorsToInclude
	^ #( counterJs )! !

SUNamed subclass: #PRCounter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Counter'!

!PRCounter methodsFor: 'conveniance' stamp: 'lr 10/1/2008 11:08'!
callback: aBlock
	"Define a callback that answers a the remaining duration until expiry."
	
	| url |
	url := canvas context actionUrl copy.
	url addParameter: (canvas callbacks registerAjaxCallback: [
		self session returnResponse: (WAResponse new
			nextPutAll: aBlock value asJavascript;
			contentType: 'text/plain';
			doNotCache;
			yourself) ]).
	self duration: aBlock value.
	self url: url! !

!PRCounter methodsFor: 'options' stamp: 'lr 10/1/2008 10:56'!
default: aString
	"The default template to be used."
	
	self optionAt: 'default' put: aString! !

!PRCounter methodsFor: 'defaults' stamp: 'lr 10/1/2008 11:33'!
defaultPrototype
	^ 'new Counter'! !

!PRCounter methodsFor: 'options' stamp: 'lr 10/1/2008 10:58'!
duration: aDuration
	"The duration until the counter expires."
	
	self optionAt: 'duration' put: aDuration! !

!PRCounter methodsFor: 'options' stamp: 'lr 10/1/2008 10:56'!
expired: aString
	"The expired template to be used."
	
	self optionAt: 'expired' put: aString! !

!PRCounter methodsFor: 'options' stamp: 'lr 10/1/2008 10:58'!
interval: aDuration
	"The interval in which the display is updated."
	
	self optionAt: 'interval' put: aDuration! !

!PRCounter methodsFor: 'options' stamp: 'lr 10/1/2008 10:58'!
sync: aDuration
	"The interval in which the duration is synced back with the server."
	
	self optionAt: 'sync' put: aDuration! !

!PRCounter methodsFor: 'options' stamp: 'lr 10/1/2008 10:59'!
url: aString
	"The url to contact to get an updated duration."
	
	self optionAt: 'url' put: aString! !

!WARenderCanvas methodsFor: '*pier-counter' stamp: 'lr 10/1/2008 11:10'!
counter
	^ PRCounter on: self! !

PRWidget subclass: #PRCountdown
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Counter'!

!PRCountdown class methodsFor: 'accessing-descriptions' stamp: 'lr 10/1/2008 11:41'!
descriptionDefault
	^ MAStringDescription new
		default: '#{days} days #{hours} hours #{minutes} minutes #{seconds} seconds';
		label: 'Default Template';
		accessor: #default;
		priority: 110;
		yourself! !

!PRCountdown class methodsFor: 'accessing-descriptions' stamp: 'lr 10/1/2008 11:25'!
descriptionExpired
	^ MAStringDescription new
		label: 'Expired Template';
		accessor: #expired;
		default: 'Done';
		priority: 120;
		yourself! !

!PRCountdown class methodsFor: 'accessing-descriptions' stamp: 'lr 10/1/2008 11:24'!
descriptionTimeout
	^ MATimeStampDescription new
		accessor: #tomeout;
		label: 'Timeout';
		priority: 100;
		beRequired;
		yourself! !

!PRCountdown class methodsFor: 'testing' stamp: 'lr 10/1/2008 10:52'!
isAbstract
	^ false! !

!PRCountdown class methodsFor: 'accessing' stamp: 'lr 10/1/2008 10:53'!
label
	^ 'Countdown'! !

!PRCountdown methodsFor: 'as yet unclassified' stamp: 'lr 10/1/2008 11:24'!
renderContentOn: html
	| timeout |
	timeout := self read: #descriptionTimeout.
	timeout isNil
		ifTrue: [ html text: 'Please choose a timeout' ]
		ifFalse: [
			html span 
				script: (html counter
					default: (self read: #descriptionDefault);
					expired: (self read: #descriptionExpired);
					callback: [ (self read: #descriptionTimeout) - TimeStamp now ]) ]! !
